#!/usr/bin/env python
"""
  #############################################################################
  #
  #     This Cplant(TM) source code is the property of Sandia National
  #     Laboratories.
  #
  #     This Cplant(TM) source code is copyrighted by Sandia National
  #     Laboratories.
  #
  #     The redistribution of this Cplant(TM) source code is subject to the
  #     terms of the GNU Lesser General Public License
  #     (see cit/LGPL or http://www.gnu.org/licenses/lgpl.html)
  #
  #     Cplant(TM) Copyright 1998, 1999, 2000, 2001, 2002 Sandia Corporation.
  #     Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
  #     license for use of this work by or on behalf of the US Government.
  #     Export of this program may require a license from the United States
  #     Government.
  #
  #############################################################################
File    : graph.py
Revision: $Revision: 1.3 $
Author  : William McLendon
Org.    : Sandia National Laboratories
Desc    : A simple directed graph.
"""

class Edge:
	def __init__(self):
		self.data = {}

class Vertex:
	def __init__(self, name):
		self.name = name
		self.data = {}
		self.indegree = 0
		self.outdegree= 0

class DiGraph:
	def __init__(self):
		self.V = {}
		self.E = {}

	def add_vertex(self, name):
		vt = Vertex(name)
		self.V[name] = vt
	
	def del_vertex(self, name):
		for ei in self.E.keys():
			if ei[0]==name or ei[1]==name:
				self.del_edge(ei[0],ei[1])
		del self.V[name]

	def del_allverts(self):
		for vi in self.V.keys():
			self.del_vertex(vi)

	def add_edge(self,source,destination):
		e1 = Edge()
		self.E[(source,destination)] = e1
		self.V[source].outdegree += 1
		self.V[destination].indegree += 1

	def add_edgedata(self,v1,v2,key,data):
		''' add_edgedata(v1, v2, key, data) '''
		self.E[(v1,v2)].data[key] = data

	def del_edgedata(self,v1,v2,key):
		del self.E[(v1,v2)].data[key]

	def del_edge(self,v1,v2):
		del self.E[(v1,v2)]
		self.V[v1].outdegree -= 1
		self.V[v2].indegree -= 1

        def del_alledges(self):
                for ei in self.E.keys():
                        self.del_edge(ei[0],ei[1])

	def clear(self):
		self.del_alledges()
		self.del_allverts()

	def reverse(self):
		for ei in self.E.keys():
			v1 = ei[0]
			v2 = ei[1]
			self.add_edge(v2,v1)
			self.E[(v2,v1)].data.update(self.E[ei].data)
			self.del_edge(v1,v2)

	def get_neighbors(self,v1name):
		''' return the downwind neighbors '''
		n = []
		if self.V.has_key(v1name):
			for ei in self.E.keys():
				if ei[0] == v1name:
					n.append(ei[1])
		return n

	def topological_sort(self):
		''' topological sort, return a list of vertex keys
		    Assume ACYCLIC graph'''
		L = []
		Q = []
		for vi in self.V.keys():
			if self.V[vi].indegree == 0:
				Q.append(self.V[vi].name)
		subEdges = {}
		while len(Q) > 0:
			v1name = Q.pop(0)
			L.append(v1name)
			N = self.get_neighbors(v1name)
			for Ni in N:
				if subEdges.has_key(Ni):
					subEdges[Ni] += 1
				else:
					subEdges[Ni] = 1
				if (self.V[Ni].indegree - subEdges[Ni]) == 0:
					Q.append(Ni)
		return L

	def Print(self):
		for vk in self.V.keys():
			print self.V[vk].__dict__
		for ek in self.E.keys():
			print "%s->%s"%ek, self.E[ek].data

# EOF
