#include <ssslib.h>
#include <stdio.h>
#include <string.h>

void SSS_perror(int sss_errno, char *prefix){
  char errbuf[MAXERRLENGTH+1];
  int type;
  
  SSS_strerror(sss_errno,errbuf,&type);

  printf("%s: %s\n",prefix,errbuf);

}

void SSS_strerror(int errnum,char *errbuf,int *type){
  switch (errnum){
  case SSS_XML_PARSE:
    *type = SSS_PARSE_ERROR;
    strncpy(errbuf,"Error parsing XML",MAXERRLENGTH);
    break;
  case SSS_NO_ROUTE:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"No route to host",MAXERRLENGTH);
    break;
  case SSS_SD_FAIL:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Error in communication with service directory",MAXERRLENGTH);
    break;
  case SSS_PROT_NAME_LONG:
    *type = SSS_PROTOCOL_ERROR;
    strncpy(errbuf,"Protocol name longer than 124 characters",MAXERRLENGTH);
    break;
  case SSS_NO_SUCH_CONN:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"A connection by that handle does not exist",MAXERRLENGTH);
    break;
  case SSS_PROTO_NO_SUPPORT:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Unsupported protocol",MAXERRLENGTH);
    break;
  case SSS_FAM_NO_SUPPORT:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Unsupported family",MAXERRLENGTH);
    break;
  case SSS_NOMEM:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Out of memory",MAXERRLENGTH);
    break;
  case SSS_NOSOCKMEM:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"No memory to make a new socket",MAXERRLENGTH);
    break;
  case SSS_INVALPROT:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Invalid protocol",MAXERRLENGTH);
    break;
  case SSS_HOST_NOT_FOUND:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Host not found",MAXERRLENGTH);
    break;
  case SSS_NO_ADDRESS:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"No address given",MAXERRLENGTH);
    break;
  case SSS_UNRECOVERABLE_DNS_ERROR:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Unrecoverable DNS error",MAXERRLENGTH);
    break;
  case SSS_TRY_AGAIN:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Temporary DNS failure - try again",MAXERRLENGTH);
    break;
  case SSS_BAD_FD:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Bad file descriptor",MAXERRLENGTH);
    break;
  case SSS_FAULT:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Memory fault",MAXERRLENGTH);
    break;
  case SSS_ALREADY_CONNECTED:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Socket is already connected",MAXERRLENGTH);
    break;
  case SSS_CONN_REFUSED:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Connection refused",MAXERRLENGTH);
    break;
  case SSS_TIMEOUT:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"Connection timed out",MAXERRLENGTH);
    break;
  case SSS_NET_UNREACH:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"Network unreachable",MAXERRLENGTH);
    break;
  case SSS_ADDR_IN_USE:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Address already in use",MAXERRLENGTH);
    break;
  case SSS_NO_FREE_PORTS:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"No free ports",MAXERRLENGTH);
    break;
  case SSS_NO_CONNECT_PERM:
    *type = SSS_CONNECT_ERROR;
    strncpy(errbuf,"No permissions to connect",MAXERRLENGTH);
    break;
  case SSS_NOSOCKPERM:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"No permissions to make socket",MAXERRLENGTH);
    break;
  case SSS_NOT_A_SOCKET:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"FD passed in is not a socket",MAXERRLENGTH);
    break;
  case SSS_SOCK_NOT_CONNECTED:
    *type = SSS_SOCKET_ERROR;
    strncpy(errbuf,"Socket is not connected",MAXERRLENGTH);
    break;
  case SSS_INVALID_SIZE:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Invalid size",MAXERRLENGTH);
    break;
  case SSS_INTERRUPT:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Interrupt",MAXERRLENGTH);
    break;
  case SSS_INVAL_ARG:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Invalid argument",MAXERRLENGTH);
    break;
  case SSS_BROKEN_PIPE:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"Broken pipe",MAXERRLENGTH);
    break;
  case SSS_AUTH_FAILED:
    *type = SSS_PROTOCOL_ERROR;
    strncpy(errbuf,"Authentication error",MAXERRLENGTH);
    break;
  case SSS_LIBRARY_ERROR:
    *type = SSS_PROTOCOL_ERROR;
    snprintf(errbuf,MAXERRLENGTH,"Library module error");
    break;
  case SSS_EOF:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"EOF on read",MAXERRLENGTH);
    break;
  case SSS_PROT_ERROR:
    *type = SSS_PROTOCOL_ERROR;
    strncpy(errbuf,"Internal error in protocol",MAXERRLENGTH);
    break;
  case SSS_NEED_MORE_INFO:
    *type = SSS_PROTOCOL_ERROR;
    strncpy(errbuf,"Protocol needs more configuration information",MAXERRLENGTH);
    break;
  case SSS_RECV_TIMEOUT:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"Message receive timed out",MAXERRLENGTH);
    break;
  case SSS_CONN_RESET:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"Connection reset by peer",MAXERRLENGTH);
    break;
  case SSS_ILLEGAL_SEEK:
    *type = SSS_COMMUNICATION_ERROR;
    strncpy(errbuf,"Illegal seek",MAXERRLENGTH);
    break;
  case SSS_SD_LOOKUP_ERROR:
    *type = SSS_LOOKUP_ERROR;
    strncpy(errbuf,"No such component",MAXERRLENGTH);
    break;
  case SSS_PERMISSION_DENIED:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"Permission denied",MAXERRLENGTH);
    break;
  case SSS_NAMETOOLONG:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"File name too long",MAXERRLENGTH);
    break;
  case SSS_FILENOTFOUND:
    *type = SSS_INTERNAL_ERROR;
    strncpy(errbuf,"File not found",MAXERRLENGTH);
    break;
  default:
    *type = SSS_UNKNOWN_ERROR;
    strncpy(errbuf,"Unknown internal error",MAXERRLENGTH);
    break;
  }
}
