#include<mod.h>
#include<ltdl.h>
#include<config.h>
#include<stdio.h>
#include<string.h>

int mod_serv_connect(struct modules *m,int extra){
  int rc;
  
  rc = m->transport->serv_connect(m->transport->state,extra);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->transport->state,sizeof(int));
    return -1;
  }
  rc = m->auth->serv_auth(m->auth->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->auth->state,sizeof(int));
    return -1;
  }
  return 0;
}

int mod_cli_connect(struct modules *m,char *host,char *extra){
  int rc;

  rc = m->transport->cli_connect(m->transport->state,host,extra);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->transport->state,sizeof(int));
    return -1;
  }
  rc = m->auth->cli_auth(m->auth->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->auth->state,sizeof(int));
    return -1;
  }
  return 0;
}

int mod_send_message(struct modules *m,char *msg){
  int rc;
  int newlen;
  char *mangled;

  mangled = m->mangling->mangle_message(m->mangling->state,msg,strlen(msg),&newlen);
  if (!mangled){
    memcpy(&(m->sss_errno),m->mangling->state,sizeof(int));
    return -1;
  }
  rc = m->framing->send_message(m->framing->state,mangled,newlen);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->framing->state,sizeof(int));
    return -1;
  }
  free(mangled);
  return 0;
}

char *mod_recv_message(struct modules *m){
  int len;
  int newlen;
  char *mangled;
  char *unmangled;

  mangled = m->framing->recv_message(m->framing->state,&len);
  if (!mangled){
    memcpy(&(m->sss_errno),m->framing->state,sizeof(int));
    return NULL;
  }
  unmangled = m->mangling->unmangle_message(m->mangling->state,mangled,len,&newlen);
  free(mangled);
  if (!unmangled){
    memcpy(&(m->sss_errno),m->mangling->state,sizeof(int));
    return NULL;
  }
  unmangled = realloc(unmangled,newlen+1);
  unmangled[newlen] = '\0';
  return unmangled;
}

int mod_close(struct modules *m){
  int rc;

  rc = m->transport->close(m->transport->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->transport->state,sizeof(int));
    return -1;
  }
  rc = m->auth->close(m->auth->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->transport->state,sizeof(int));
    return -1;
  }
  rc = m->framing->close(m->framing->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->framing->state,sizeof(int));
    return -1;
  }
  rc = m->mangling->close(m->mangling->state);
  if (rc == -1){
    memcpy(&(m->sss_errno),m->mangling->state,sizeof(int));
    return -1;
  }
  lt_dlclose(m->transport->handle);
  free(m->transport);
  lt_dlclose(m->auth->handle);
  free(m->auth);
  lt_dlclose(m->framing->handle);
  free(m->framing);
  lt_dlclose(m->mangling->handle);
  free(m->mangling);
  free(m);
  return 0;
}

struct modules *mod_init(SSS_ConfigDict cfg,int debug,char *transport,char *framing,char *auth,char *mangling){
  struct modules *ret;
  int modstr_len;
  char *modstr;
  char snbuf[1];
  
  void *(*transport_getstate)(SSS_ConfigDict,int);
  void *(*auth_getstate)(SSS_ConfigDict,int,void *);
  void *(*framing_getstate)(SSS_ConfigDict,int,void *);
  void *(*mangling_getstate)(SSS_ConfigDict,int);

  ret = malloc(sizeof(struct modules));
  ret->transport = malloc(sizeof(struct transport));
  ret->auth = malloc(sizeof(struct auth));
  ret->framing = malloc(sizeof(struct framing));
  ret->mangling = malloc(sizeof(struct mangling));

  modstr_len = snprintf(snbuf,1,"%s/transport_%s",MODPATH,transport);
  modstr = malloc(modstr_len + 1);

  lt_dlinit();

  sprintf(modstr,"%s/transport_%s",MODPATH,transport);
  ret->transport->handle = lt_dlopenext(modstr);
  if (!ret->transport->handle){
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  free(modstr);
  transport_getstate = (void *(*)(SSS_ConfigDict,int))lt_dlsym(ret->transport->handle,"transport_getstate");
  if (!transport_getstate){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->write = (int (*)(void *,void *,int))lt_dlsym(ret->transport->handle,"transport_write");
  if (!ret->transport->write){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->cli_connect = (int (*)(void *,char *,char *))lt_dlsym(ret->transport->handle,"cli_connect");
  if (!ret->transport->cli_connect){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->serv_connect = (int (*)(void *,int))lt_dlsym(ret->transport->handle,"serv_connect");
  if (!ret->transport->serv_connect){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->writen = (int (*)(void *,void *,int))lt_dlsym(ret->transport->handle,"transport_writen");
  if (!ret->transport->writen){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->read = (int (*)(void *,void *,int))lt_dlsym(ret->transport->handle,"transport_read");
  if (!ret->transport->read){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->readn = (int (*)(void *,void *,int))lt_dlsym(ret->transport->handle,"transport_readn");
  if (!ret->transport->readn){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->close = (int (*)(void *))lt_dlsym(ret->transport->handle,"transport_close");
  if (!ret->transport->close){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->transport->state = transport_getstate(cfg,debug);
  if (!ret->transport->state){
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  
  modstr_len = snprintf(snbuf,1,"%s/auth_%s",MODPATH,auth);
  modstr = malloc(modstr_len + 1);

  sprintf(modstr,"%s/auth_%s",MODPATH,auth);

  ret->auth->handle = lt_dlopenext(modstr);
  if (!ret->auth->handle){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  free(modstr);
  auth_getstate = (void *(*)(SSS_ConfigDict,int,void *))lt_dlsym(ret->auth->handle,"auth_getstate");
  if (!auth_getstate){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }    
  ret->auth->cli_auth = (int (*)(void *))lt_dlsym(ret->auth->handle,"cli_auth");
  if (!ret->auth->cli_auth){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->auth->serv_auth = (int (*)(void *))lt_dlsym(ret->auth->handle,"serv_auth");
  if (!ret->auth->serv_auth){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->auth->close = (int (*)(void *))lt_dlsym(ret->auth->handle,"auth_close");
  if (!ret->auth->close){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->auth->state = auth_getstate(cfg,debug,ret->transport);
  if (!ret->auth->state){
    ret->transport->close(ret->transport->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  
  modstr_len = snprintf(snbuf,1,"%s/framing_%s",MODPATH,framing);
  modstr = malloc(modstr_len + 1);

  sprintf(modstr,"%s/framing_%s",MODPATH,framing);

  ret->framing->handle = lt_dlopenext(modstr);
  if (!ret->framing->handle){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  free(modstr);
  framing_getstate = (void *(*)(SSS_ConfigDict,int,void *))lt_dlsym(ret->framing->handle,"framing_getstate");
  if (!framing_getstate){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->framing->send_message = (int (*)(void *,void *,int))lt_dlsym(ret->framing->handle,"send_message");
  if (!ret->framing->send_message){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->framing->recv_message = (char *(*)(void *,int *))lt_dlsym(ret->framing->handle,"recv_message");
  if (!ret->framing->recv_message){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->framing->close = (int (*)(void *))lt_dlsym(ret->framing->handle,"framing_close");
  if (!ret->framing->close){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->framing->state = framing_getstate(cfg,debug,ret->transport);
  if (!ret->framing->state){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }

  modstr_len = snprintf(snbuf,1,"%s/mangling_%s",MODPATH,mangling);
  modstr = malloc(modstr_len + 1);

  sprintf(modstr,"%s/mangling_%s",MODPATH,mangling);

  ret->mangling->handle = lt_dlopenext(modstr);
  if (!ret->mangling->handle){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    ret->framing->close(ret->framing->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  free(modstr);
  mangling_getstate = (void *(*)(SSS_ConfigDict,int))lt_dlsym(ret->mangling->handle,"mangling_getstate");
  if (!mangling_getstate){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    lt_dlclose(ret->mangling->state);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->mangling->mangle_message = (char *(*)(void *,char *,int,int *))lt_dlsym(ret->mangling->handle,"mangle_message");
  if (!ret->mangling->mangle_message){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    ret->framing->close(ret->framing->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    lt_dlclose(ret->mangling->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->mangling->unmangle_message = (char *(*)(void *,char *,int,int *))lt_dlsym(ret->mangling->handle,"unmangle_message");
  if (!ret->mangling->unmangle_message){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    ret->framing->close(ret->framing->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    lt_dlclose(ret->mangling->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->mangling->close = (int (*)(void *))lt_dlsym(ret->mangling->handle,"mangling_close");
  if (!ret->mangling->close){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    ret->framing->close(ret->framing->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    lt_dlclose(ret->mangling->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }
  ret->mangling->state = mangling_getstate(cfg,debug);
  if (!ret->mangling->state){
    ret->transport->close(ret->transport->state);
    ret->auth->close(ret->auth->state);
    ret->framing->close(ret->framing->state);
    lt_dlclose(ret->transport->handle);
    lt_dlclose(ret->auth->handle);
    lt_dlclose(ret->framing->handle);
    lt_dlclose(ret->mangling->handle);
    free(ret->transport);
    free(ret->auth);
    free(ret->framing);
    free(ret->mangling);
    free(ret);
    return NULL;
  }

  return ret;
}
